#include "envy.h"
#include "envy_config.h"
#include "envy_osdb.h"
#include "envy_options.h"
#include <pthread.h>
#include <libgen.h>

static void _print_usage(const char *cmd);

const char * 
envy_parse_opt(int argc, char **argv, Envy_Config *ec)
{
  char c;
  struct option long_options[] =
    {
      {"sub",     required_argument, 0, 's'},
      {"osdb",          no_argument, 0, 'o'},
      {"help",          no_argument, 0, 'h'},
      { 0,              no_argument, 0,  0 }
    };
  int option_index = 0;
  int osdb = 0;
  char *subfile = NULL;

  while (1)
    {
      Envy_Osdb *eo;
      c = getopt_long(argc, argv, "s:oh",
                      long_options, &option_index);
  
      if (c == -1)
        break;
  
      switch (c)
        {
        case 's':
          subfile = optarg;
          break;
        case 'o':
          osdb = 1;
          break;
        case 'h':
          _print_usage(argv[0]);
          exit(EXIT_SUCCESS);
          break;
        default:
          _print_usage(argv[0]);
          exit(EXIT_FAILURE);
        }
    }

  envy->filename = argv[argc-1];
  envy->dirname = strdup(envy->filename);
  envy->dirname = dirname(envy->dirname);

  if (!subfile)
    {
      char *p;
      subfile = calloc(strlen(envy->filename)+1, sizeof(char));
      strcpy(subfile, envy->filename);
      p = &subfile[strlen(subfile) - 1];
      while (p)
        {
          p--;
          if (*p == '/') break;
          if (*p == '.')
            {
              p++;
              *p = '\0';
              strcat(subfile, "srt");
              if (!access(subfile, R_OK))
                {
                  envy_load_subtitle(subfile);
                  return envy->filename;
                }
              else
                {
                  free(subfile);
                  subfile = NULL;
                  break;
                }
            }
        }
    }
  else
    {
      envy_load_subtitle(subfile);
    }

  if (osdb || (ec && ec->auto_osdb))
    {
      pthread_t thread;
      Envy_Osdb *eo;

      /* Setting Up OSDB params */
      eo = envy_osdb_new("http://www.opensubtitles.org/xml-rpc");
      if (*(ec->langage) != '\0') strncpy(eo->lang, ec->langage, 3);
      eo->keep_osdb_subfilename = ec->keep_osdb_subfilename;

      char *ext;
      eo->subfilename = calloc(strlen(envy->filename) + 1, sizeof(char));
      strcpy(eo->subfilename, envy->filename);
      ext = strrchr(eo->subfilename, '.');
      if (ext != NULL)
        {
          strcpy(ext, ".srt");
        }
      else
        {
          free(eo->subfilename);
          eo->subfilename = NULL;
        }

      envy_moviesize_get(eo, envy->filename);
      envy_moviehash_get(eo, envy->filename);

      pthread_create(&thread, NULL, (void *(*)(void *))envy_osdb_process, eo);
      pthread_detach(thread);
    }

  return argv[argc-1];
}

static void 
_print_usage(const char *cmd)
{
  printf("Usage: %s [-oh] [-s subfile] filename\n", cmd);
  printf("  -s, --sub      "
         "Load subfile for the subtitles \n");
  printf("  -o, --osdb     "
         "Try to download the subfile on OpenSubtitles.org\n");
  printf("  -h, --help     "
         "Affiche ce message\n");
}
